/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "ModulesPicker.h"

#include <Modules/Splitter/Splitter.h>

#include <QDebug>
#include <QPushButton>

ModulesPicker::ModulesPicker() { setupUi(); }

void ModulesPicker::setupUi()
{
    mainLayout = new QGridLayout;
    setLayout(mainLayout);

    dataVisualizationGroupBox = new QGroupBox("Data Visualization");
    mainLayout->addWidget(dataVisualizationGroupBox, 0, 0);
    dataVisualizationList = new QVBoxLayout();
    dataVisualizationList->addStretch();
    dataVisualizationGroupBox->setLayout(dataVisualizationList);

    dataSourcesGroupBox = new QGroupBox("Data Sources");
    mainLayout->addWidget(dataSourcesGroupBox, 1, 0);
    dataSourcesList = new QVBoxLayout();
    dataSourcesList->addStretch();
    dataSourcesGroupBox->setLayout(dataSourcesList);

    utilitiesGroupBox = new QGroupBox("Utilities");
    mainLayout->addWidget(utilitiesGroupBox, 0, 1);
    utilitiesList = new QVBoxLayout();
    utilitiesList->addStretch();
    utilitiesGroupBox->setLayout(utilitiesList);

    organizationGroupBox = new QGroupBox("Organization");
    mainLayout->addWidget(organizationGroupBox, 1, 1);
    organizationList = new QVBoxLayout();
    organizationList->addStretch();
    organizationGroupBox->setLayout(organizationList);

    testingGroupBox = new QGroupBox("Testing");
    mainLayout->addWidget(testingGroupBox, 0, 2);
    testingList = new QVBoxLayout();
    testingList->addStretch();
    testingGroupBox->setLayout(testingList);

    for (ModuleInfo moduleInfo : ModulesList::getInstance().getModulesInfo())
    {
        switch (moduleInfo.getModuleCategory())
        {
            case ModuleCategory::DATA_VISUALIZATION:
                dataVisualizationList->insertWidget(
                    0, createButton(moduleInfo.getModuleName()));
                break;
            case ModuleCategory::DATA_SOURCES:
                dataSourcesList->insertWidget(
                    0, createButton(moduleInfo.getModuleName()));
                break;
            case ModuleCategory::UTILITIES:
                utilitiesList->insertWidget(
                    0, createButton(moduleInfo.getModuleName()));
                break;
            case ModuleCategory::ORGANIZATION:
                organizationList->insertWidget(
                    0, createButton(moduleInfo.getModuleName()));
                break;
            case ModuleCategory::TESTING:
                testingList->insertWidget(
                    0, createButton(moduleInfo.getModuleName()));
                break;
            default:
                break;
        }
    }
}

QPushButton* ModulesPicker::createButton(const QString& name)
{
    QPushButton* button = new QPushButton(name);
    connect(button, &QPushButton::clicked, this,
            [this, button]()
            {
                ModuleId id;
                if (ModulesList::getInstance().getModuleId(button->text(), id))
                {
                    emit onModuleSelected(id);
                }

                close();
            });

    return button;
}
