/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "SaveConfigurationDialog.h"

#include <QFileDialog>

#include "ui_SaveConfigurationDialog.h"

SaveConfigurationDialog::SaveConfigurationDialog(QWidget *parent)
    : QDialog(parent), ui(new Ui::SaveConfigurationDialog)
{
    ui->setupUi(this);

    connect(ui->pushButton_saveConfig, &QPushButton::clicked, this,
            &SaveConfigurationDialog::onSaveConfigClicked);
    connect(ui->pushButton_selectIcon, &QPushButton::clicked, this,
            &SaveConfigurationDialog::onSelectIconPathClicked);
}

SaveConfigurationDialog::~SaveConfigurationDialog() { delete ui; }

void SaveConfigurationDialog::pickConfigData(QString *name, QString *iconPath,
                                             QString *description)
{
    this->name        = name;
    this->iconPath    = iconPath;
    this->description = description;

    this->exec();
}

QString SaveConfigurationDialog::getConfigName() const
{
    return ui->textEdit_configName->text().trimmed();
}

QString SaveConfigurationDialog::getConfigIconPath() const
{
    return ui->textEdit_iconPath->toPlainText().trimmed();
}

QString SaveConfigurationDialog::getConfigDescription() const
{
    return ui->textEdit_description->toPlainText().trimmed();
}

void SaveConfigurationDialog::setConfigName(const QString &value)
{
    ui->textEdit_configName->setText(value);
}

void SaveConfigurationDialog::setConfigIconPath(const QString &value)
{
    ui->textEdit_iconPath->setText(value);
}

void SaveConfigurationDialog::setConfigDescription(const QString &value)
{
    ui->textEdit_description->setText(value);
}

void SaveConfigurationDialog::onSelectIconPathClicked()
{
    QString filePath = QFileDialog::getOpenFileName(
        nullptr, "Select config icon", "", "(*.jpeg, *.png., *.jpg)");
    setConfigIconPath(filePath);
}

void SaveConfigurationDialog::onSaveConfigClicked()
{
    (*name)        = getConfigName();
    (*iconPath)    = getConfigIconPath();
    (*description) = getConfigDescription();
    this->close();
}
