/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "Topic.h"

#include <QRegExp>

Topic::Topic() : topic("") {}

Topic::Topic(QString topic) : topic(topic) {}

bool Topic::operator==(const Topic& toCompare) const
{
    // Always true if the current topic is a pure wildcard
    if (topic == "*" || topic == "?")
        return true;

    // If the current topic isn't a wildcard then the topic must match
    if (!isWildcard())
        return topic == toCompare.toString();

    // If the current topic is a wildcard then only the prefix must match
    return toCompare.toString().startsWith(getPrefix().toString());
}

bool Topic::operator<(const Topic& toCompare) const
{
    return topic < toCompare.topic;
}

Topic Topic::getParent() const
{
    int idx = topic.lastIndexOf('/');
    return idx == -1 ? Topic(topic) : Topic(topic.left(idx));
}

Topic Topic::getRoot() const
{
    int idx = topic.indexOf('/');
    return idx == -1 ? Topic(topic) : Topic(topic.left(idx));
}

Topic Topic::getPrefix() const
{
    if (!isWildcard())
        return *this;

    int idxAst = topic.lastIndexOf('*');
    int idxQue = topic.lastIndexOf('?');

    int idx = idxAst != -1 ? idxAst : idxQue;

    return Topic(topic.left(idx));
}

bool Topic::isValid() const
{
    return topic.contains(QRegExp("^[A-Za-z0-9$_]+(\\/[A-Za-z0-9$_]+)*$"));
}

bool Topic::isWildcard() const
{
    // Check wether the last character is a wildcard
    return topic.at(topic.size() - 1) == '*' ||
           topic.at(topic.size() - 1) == '?';
}

QString Topic::toString() const { return topic; }
