/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QString>

/**
 * @brief Topics are strings made of alphanumerical characters and slashes '/'.
 *
 * For example, "Prefix/Message" and "SomeMessage" are valid topics.
 */
class Topic
{
public:
    Topic();
    explicit Topic(QString topic);

    /**
     * @brief Check if the given topic matches, also if the current is a
     * wildcard.
     */
    bool operator==(const Topic& toCompare) const;

    bool operator<(const Topic& toCompare) const;

    /**
     * @brief Returns the parent.
     *
     * If the topic is Prefix/Message, returns Prefix.
     */
    Topic getParent() const;

    /**
     * @brief Returns the topic root.
     *
     * If the topic is Topic/SubTopic/SubSubTopic, returns Topic.
     */
    Topic getRoot() const;

    /**
     * @brief Removes the wildcard from a wildcard topic.
     */
    Topic getPrefix() const;

    /**
     * @brief True if the topic is valid.
     */
    bool isValid() const;

    /**
     * @brief Returns true if the topic is a wildcard.
     */
    bool isWildcard() const;

    QString toString() const;

private:
    QString topic;
};
