/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "MessageBroker.h"

#include <Modules/Module.h>

#include "Subscription.h"

MessageBroker& MessageBroker::getInstance()
{
    static MessageBroker instance;
    return instance;
}

std::unique_ptr<Subscription> MessageBroker::subscribe(
    Filter filter, MessageBroker::SubscriberFn callback)
{
    auto subscription = new Subscription(std::move(callback), filter);

    observers.insert(filter, subscription);

    return std::unique_ptr<Subscription>(subscription);
}

void MessageBroker::unsubscribe(Subscription* sub)
{
    auto filter       = sub->filter;
    auto callbackList = observers.values(filter);

    for (auto& elem : callbackList)
    {
        if (elem == sub)
            observers.remove(filter, elem);
    }
}

void MessageBroker::publish(const Message& message)
{
    // Iterate over all filters to check if the message matches any
    for (auto filter : observers.uniqueKeys())
    {
        // If the message matches this particular filter, then notify all the
        // subscribers
        if (filter.match(message))
        {
            auto filteredSubscribers = observers.values(filter);
            for (auto subscriber = filteredSubscribers.begin();
                 subscriber != filteredSubscribers.end();)
            {

                auto copy = message;
                filter.filterMessage(copy);
                (*subscriber)->callback(copy, filter);

                subscriber++;
            }
        }
    }
}
