/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Core/Message/Filter.h>

#include <QObject>
#include <functional>
#include <memory>

class Subscription;

class MessageBroker : public QObject
{
    Q_OBJECT

public:
    static MessageBroker& getInstance();
    using SubscriberFn = std::function<void(const Message&, const Filter&)>;

    [[nodiscard]] std::unique_ptr<Subscription> subscribe(
        Filter filter, MessageBroker::SubscriberFn callback);

    /**
     * @brief Publish a message to all its subscribers.
     */
    void publish(const Message& message);

    friend class Subscription;

private:
    /**
     * @brief Removes the subscription from the filter.
     */
    void unsubscribe(Subscription* sub);

    MessageBroker(){};

    QMultiMap<Filter, Subscription*> observers;
};
