/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Core/XmlObject.h>
#include <Modules/Module.h>

#include <QCloseEvent>
#include <QMainWindow>
#include <QMenu>
#include <memory>

#include "Components/ErrorDisplayer/ErrorDisplayer.h"

class Window : public QMainWindow
{
    Q_OBJECT

public:
    explicit Window(XmlObject configuration);

    XmlObject toXmlObject();

    std::unique_ptr<ErrorDisplayer> errorDisplayer;

    void closeEvent(QCloseEvent* event) override;

signals:
    void onWindowClosed();

private:
    std::shared_ptr<Module> child;
    void setChild(std::shared_ptr<Module> module);

private slots:
    void replaceChild(Module* const oldModule,
                      std::shared_ptr<Module> newModule);
    void closeChild(Module* module);
};
