/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "CommandPad.h"

#include <Core/MessageBroker/MessageBroker.h>
#include <Modules/CommandPad/MessagesList.h>

#include <QPushButton>
#include <QVBoxLayout>

CommandPad::CommandPad() : Module(ModuleId::COMMAND_PAD) { setupUi(); }

CommandPad::~CommandPad() {}

XmlObject CommandPad::toXmlObject()
{
    XmlObject obj = Module::toXmlObject();

    auto key = messagesListComboBox->currentText();
    if (formElements.find(key) != formElements.cend())
    {
        obj.addAttribute("message_id", key);
        formElements[key]->toXmlObject(obj);
    }

    return obj;
}

void CommandPad::fromXmlObject(const XmlObject &obj)
{
    auto key = obj.getAttribute("message_id");

    if (formElements.find(key) != formElements.cend())
    {
        formElements[key]->fromXmlObject(obj);
        messagesListComboBox->setCurrentText(key);
    }
}

void CommandPad::setupUi()
{
    QVBoxLayout *outerLayout = new QVBoxLayout;
    outerLayout->addStretch();

    messagesListComboBox = std::make_unique<QComboBox>();
    messagesListComboBox->setSizePolicy(
        QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
    messagesListComboBox->addItems(MessagesList::messagesList);
    outerLayout->addWidget(messagesListComboBox.get());

    QGridLayout *formGridLayout = new QGridLayout;
    outerLayout->addLayout(formGridLayout);

    QPushButton *sendButton = new QPushButton("Send");
    outerLayout->addWidget(sendButton);

    setLayout(outerLayout);

    MessagesList::fillMessagesMap(formElements);

    // Set default value
    currentMessage = "PING_TC";
    messagesListComboBox->setCurrentText(currentMessage);

    connect(messagesListComboBox.get(), &QComboBox::currentTextChanged, this,
            [=](QString key)
            {
                if (formElements.find(key) != formElements.cend())
                {
                    formElements[currentMessage]->removeFromGridLayout(
                        formGridLayout);
                    formElements[key]->applyToGridLayout(formGridLayout);
                    currentMessage = key;
                }
            });

    connect(sendButton, &QPushButton::clicked,
            [=]()
            {
                auto key = messagesListComboBox->currentText();
                if (formElements.find(key) != formElements.cend())
                {
                    auto message = formElements[key]->prepareMessage(key);
                    MessageBroker::getInstance().publish(message);
                }
            });
}
