#pragma once

#include <Core/Message/Message.h>
#include <Core/MessageBroker/Subscription.h>
#include <Modules/Module.h>

#include <QFile>
#include <QLineEdit>
#include <QWidget>

namespace Ui
{
class FileStreamModule;
}

class FileStreamModule : public Module
{
    Q_OBJECT

public:
    FileStreamModule();
    ~FileStreamModule();

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject& xmlObject) override;

protected:
    void connectUI();
    QString getFilePath() const;
    QString getTopicName() const;
    void resetTopicName();
    void onMsgReceived(const Message& msg);

    void onAddTopicClicked();
    void onStartClicked();
    void onStopClicked();
    void onTopicEdit();
    void disableOnStartElement();
    void enableElementOnStop();
    void onFileChanged();
    void onShowFileClick();

    void addTopic(const QString& topic);
    QLineEdit* createTopicView(const QString& topic) const;

    void deleteAllTopicViews();

private:
    Ui::FileStreamModule* ui;

    std::vector<std::pair<QLineEdit*, std::unique_ptr<Subscription>>>
        topicViewsList;
    std::vector<std::unique_ptr<Subscription>> topicViewsSubscriptions;
    QFile file;
};
