/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Components/SubscriptionsPanel/SubscriptionsPanel.h>
#include <Core/Message/Filter.h>
#include <Core/MessageBroker/Subscription.h>
#include <Modules/Module.h>

#include <QTextEdit>
#include <memory>

class IncomingMessagesViewerModule : public Module
{
    Q_OBJECT

public:
    IncomingMessagesViewerModule();
    ~IncomingMessagesViewerModule();

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject& xmlObject) override;

private slots:
    void onSubscribeClicked();
    void onFilterAdded(const Filter& filter);
    void onFilterRemoved(const Filter& filter);

private:
    std::unique_ptr<QTextEdit> edit;
    void setupUi();
    void customContextMenuActionSetup();

    std::map<Filter, std::unique_ptr<Subscription>> filters;

    bool keepOnlyLastMessage = false;
    std::shared_ptr<QAction> keepOnlyLastMessageAction;
    bool useTimestamp = false;
    std::shared_ptr<QAction> useTimestampAction;

    std::unique_ptr<SubscriptionsPanel> subscriptionsPanel;
};
