/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Core/Message/Message.h>
#include <Core/MessageBroker/Subscription.h>
#include <Modules/Module.h>

#include <QWidget>
#include <memory>

class QTableWidgetItem;

class MessageLog
{

public:
    explicit MessageLog(std::shared_ptr<QTableWidgetItem> item,
                        const Message& msg);

    std::shared_ptr<QTableWidgetItem> getItem() const;
    void setItem(std::shared_ptr<QTableWidgetItem> value);

    Message getMsg() const;
    void setMsg(const Message& value);

private:
    std::shared_ptr<QTableWidgetItem> item = nullptr;
    Message msg;
};

namespace Ui
{
class OutgoingMessagesViewerModule;
}

class OutgoingMessagesViewerModule : public Module
{
    Q_OBJECT

public:
    OutgoingMessagesViewerModule();
    ~OutgoingMessagesViewerModule();

    void addMsgSent(const Message& msg);
    void handleAck(const Message& ack);
    void handleNack(const Message& nack);
    void handleWack(const Message& wack);

protected:
    int updateVerticalHeaders(const Message& msg);
    void registerMessage(std::shared_ptr<QTableWidgetItem> item,
                         const Message& msg);
    QString computeMsgName(const Message& msg);

private:
    Ui::OutgoingMessagesViewerModule* ui;
    QList<MessageLog> messages;
    int maxSize = 10;

    std::unique_ptr<Subscription> logCommandsSub;
    std::unique_ptr<Subscription> mavlinkRecvMsgACKSub;
    std::unique_ptr<Subscription> mavlinkRecvMsgNACKSub;
    std::unique_ptr<Subscription> mavlinkRecvMsgWACKSub;
};
