/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <QList>
#include <QMap>
#include <QString>

namespace RigStatesList
{

// Rockets states
enum class State : int
{
    IDLE = 0,
    INIT,
    INIT_ERR,
    DISARMED,
    ARMED,
    FIRING,
    IGNITING,
    OXIDIZER,
    COOLING,
    INVALID,
};

// Groundstation labels
static const QList<QString> statesLabels{"INIT",     "DISARMED", "ARMED",
                                         "IGNITING", "OXIDIZER", "COOLING"};

// Map from rocket states to groundstation labels
static const QMap<State, int> statesIndexes{
    {State::IDLE, -1},    {State::INIT, 0},   {State::INIT_ERR, 0},
    {State::DISARMED, 1}, {State::ARMED, 2},  {State::IGNITING, 3},
    {State::OXIDIZER, 4}, {State::COOLING, 5}};

}  // namespace RigStatesList