/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#pragma once

#include <Modules/Module.h>

#include <QSplitter>
#include <QWidget>

class Splitter : public Module
{
    Q_OBJECT

public:
    explicit Splitter(Qt::Orientation orientation, ModuleId id);

    XmlObject toXmlObject() override;
    void fromXmlObject(const XmlObject &obj) override;

    void addModule(std::shared_ptr<Module> module);
    void addModule(std::shared_ptr<Module> module, int position);
    Qt::Orientation getOrientation() const;

    void setOrientation(Qt::Orientation orientation);
    void swapOrientation();

public slots:
    void replaceChild(Module *const oldModule,
                      std::shared_ptr<Module> newModule);
    void closeChild(Module *module);

private:
    void customContextMenuActionSetup();
    void setupUi(Qt::Orientation orientation = Qt::Horizontal);

    std::unique_ptr<QSplitter> splitter;
    std::map<int, std::shared_ptr<Module>> children;
};
