/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "TestModule.h"

#include <Core/MessageBroker/MessageBroker.h>

#include <QTime>

#include "ui_TestModule.h"

TestModule::TestModule() : Module(ModuleId::BROKER_TEST), ui(new Ui::TestModule)
{
    ui->setupUi(this);

    maxChar = ui->maxChar_spinBox->value();
    connect(ui->maxChar_spinBox, SIGNAL(valueChanged(int)), this,
            SLOT(onMaxCharChanged(int)));
}

TestModule::~TestModule() { delete ui; }

// TODO: get subscriptions from xml
// XmlObject TestModule::toXmlObject()
// {
//     XmlObject obj = Module::toXmlObject();
//     return obj
// }
// void TestModule::fromXmlObject(const XmlObject& xmlObject)
// {
//     Q_UNUSED(xmlObject);
// }

void TestModule::on_publish_button_clicked()
{
    QString topic   = ui->topic_lineEdit->text().trimmed();
    QString payload = ui->payload_lineEdit->text().trimmed();
    Message message(Topic{topic});
    auto valueField = Field(payload);
    message.setField("value", valueField);
    auto timestampField =
        Field((uint64_t)QTime::currentTime().msecsSinceStartOfDay());
    message.setField("timestamp", timestampField);
    MessageBroker::getInstance().publish(message);
}

void TestModule::on_subscribe_button_clicked()
{
    subscription = MessageBroker::getInstance().subscribe(
        Filter::fromString(ui->subscribe_lineEdit->text().trimmed()),
        [this](const Message& message, const Filter& filter)
        {
            QString oldText = ui->textArea->toPlainText();
            QString line =
                QDateTime::currentDateTime().toString("hh.mm.ss (zzz): ") +
                message.toString();
            QString newText(line + '\n' + oldText);
            ui->textArea->setPlainText(newText.mid(0, maxChar));
        });
}

void TestModule::on_unsubscribe_button_clicked() { subscription = nullptr; }

void TestModule::onMaxCharChanged(int value) { maxChar = value; }
